@@ -83,13 +83,38 @@ def check_requires(fn: Any, name: str, strict: bool = True) -> bool:
         if not strict and param.kind is inspect.Parameter.VAR_KEYWORD:
             return True
         if k == name:
-            if param.kind is inspect.Parameter.KEYWORD_ONLY:
-                return True
-            if param.kind is inspect.Parameter.POSITIONAL_OR_KEYWORD:
-                return True
+            if param.kind is inspect.Parameter.VAR_POSITIONAL:
+                return False
+            return True
     return False
 
 
+def filter_kw(
+    fn: Callable,
+    kwargs: Dict[str, Any],
+    *,
+    strict: bool = False,
+) -> Dict[str, Any]:
+    kw = {}
+    for k, v in kwargs.items():
+        if check_requires(fn, k, strict):
+            kw[k] = v
+    return kw
+
+
+def get_num_positional_args(fn: Callable) -> Union[int, float]:
+    signature = inspect.signature(fn)
+    counter = 0
+    for param in signature.parameters.values():
+        if param.kind is inspect.Parameter.VAR_POSITIONAL:
+            return math.inf
+        if param.kind is inspect.Parameter.POSITIONAL_ONLY:
+            counter += 1
+        elif param.kind is inspect.Parameter.POSITIONAL_OR_KEYWORD:
+            counter += 1
+    return counter
+
+
 def prepare_workplace_from(workplace: str, timeout: timedelta = timedelta(30)) -> str:
     current_time = datetime.now()
     if os.path.isdir(workplace):
@@ -200,6 +225,10 @@ def hash_code(code: str) -> str:
     return hashlib.md5(code).hexdigest()[:8]
 
 
+def random_hash() -> str:
+    return hash_code(str(random.random()))
+
+
 def prefix_dict(d: Dict[str, Any], prefix: str):
     """Prefix every key in dict `d` with `prefix`."""
 
@@ -445,6 +474,10 @@ class WithRegister(Generic[T]):
             before_register=before,
         )
 
+    @classmethod
+    def remove(cls, name: str) -> Callable[[Type[T]], Type[T]]:
+        return cls.d.pop(name)
+
     @classmethod
     def check_subclass(cls, name: str) -> bool:
         return issubclass(cls.d[name], cls)
